/*
 *  Copyright (c) 2018 LLC LEXY. All rights reserved.
 */

#ifndef __V_RECOGNITION_MODULE_BASE_H_
#define __V_RECOGNITION_MODULE_BASE_H_

#include <chrono>
#include <vnetwork/core/v_neurocommunicator_base.h>

class VRecognitionModuleBleAdapter;

class VNETWORK_LIBRARY_EXPORT VRecognitionModuleBase : public VNeurocommunicatorBase
{
    Q_OBJECT

public:
    VRecognitionModuleBase( unsigned id,
                            const QString& applicationName,
                            std::size_t frameSize,
                            const QString& speakersCardName,
                            unsigned playbackSamplerate,
                            const std::chrono::milliseconds& hideTextTime,
                            const QString& clientXmppPassword,
                            const QString& logsFileToSend ) noexcept;

    void start( bool advertise = true ) noexcept override;

protected:
    void registerInitialParameters( quint8 powerLevel,
                                    bool sendingLogs,
                                    quint8 minFreeSpacePercent,
                                    bool useExternalMicrophone,
                                    double fontsScale,
                                    bool useDemoMode,
                                    bool saveSound,
                                    quint64 brailleDisplayMacAddress,
                                    const QString& brailleDisplayName,
                                    quint16 connectionsCount,
                                    bool pushToRecognize,
                                    bool installUpdatesAutomatically,
                                    uint8_t updateStatus,
                                    bool onlineChatEnabled,
                                    bool onlineChatCommentsEnabled,
                                    const QString& currentVersion,
                                    const QString& updateVersion,
                                    const QString& updateDescription,
                                    const QDateTime& lastSendLogTime ) noexcept;

    void setUseExternalMicrophone( bool useExternalMicrophone ) noexcept;
    void setFontsScale( double newScale ) noexcept;
    void setConnectionsCount( quint16 connectionsCount ) noexcept;
    void setPushToRecognize( bool pushToRecognize ) noexcept;
    void setUseDemoMode( bool useDemoMode ) noexcept;
    void setSaveSound( bool saveSound ) noexcept;
    void setOnlineChatEnabled( bool enabled ) noexcept;
    void setOnlineChatCommentsEnabled( bool enabled ) noexcept;

    void sendText( const QString& text ) noexcept;

    virtual bool processSetUseExternalMicrophoneRequested( bool useExternalMicrophone ) noexcept = 0;
    virtual bool processSetPushToRecognizeRequested( bool pushToRecognize ) noexcept = 0;
    virtual bool processSetFontsScaleRequested( double newScale ) noexcept = 0;
    virtual bool processSetUseDemoModeRequested( bool useDemoMode ) noexcept = 0;
    virtual bool processSetSaveSoundRequested( bool saveSound ) noexcept = 0;
    virtual bool processSetEnableOnlineChatRequested( bool enabled ) noexcept = 0;
    virtual bool processSetEnableOnlineChatCommentsRequested( bool enabled ) noexcept = 0;

    virtual void processNewSmartCaneAddressReceived( const QBluetoothAddress& address ) noexcept = 0;

    virtual void processUserTextReceived( const QString& text ) noexcept = 0;
    virtual void processClearScreen( ) noexcept = 0;

private slots:
    void onSetUseExternalMicrophoneRequested( bool useExternalMicrophone ) noexcept;
    void onSetPushToRecognizeRequested( bool pushToRecognize ) noexcept;
    void onSetFontsScaleRequested( double newScale ) noexcept;
    void onSetUseDemoModeRequested( bool useDemoMode ) noexcept;
    void onSetSaveSoundRequested( bool saveSound ) noexcept;
    void onSetEnableOnlineChatRequested( bool enable ) noexcept;
    void onSetEnableOnlineChatCommentsRequested( bool enable ) noexcept;

    void onSmartCaneAddressGot( const QBluetoothAddress& address ) noexcept;

    void onUserTextReceived( const QString& text ) noexcept;

    void checkEthernetStatus( ) noexcept;

    void onClearScreenReceived( ) noexcept;

private:
    Q_DISABLE_COPY( VRecognitionModuleBase )

    VRecognitionModuleBleAdapter* getBleAdapter( ) noexcept;

    QTimer *m_ethernetTimer;
};

#endif
