/*
 *  Copyright (c) 2018 LLC LEXY. All rights reserved.
 */

#ifndef __V_SMART_CANE_BASE_H_
#define __V_SMART_CANE_BASE_H_

#include <vnetwork/core/v_neurocommunicator_base.h>

class QUuid;
class QNetworkReply;
class QAuthenticator;

class VSmartCaneBleAdapter;

class VNETWORK_LIBRARY_EXPORT VSmartCaneBase : public VNeurocommunicatorBase
{
    Q_OBJECT

public:
    enum
    {
        MALE_VOICE = 1,
        FEMALE_VOICE
    };

    enum
    {
        BUTTON_1_STATE_INFO = 1,
        BUTTON_1_FAST_RECOGNITION,
        BUTTON_1_FACES_RECOGNITION,
        BUTTON_1_TRAFFIC_SIGNS_RECOGNITION,
        BUTTON_1_TEXT_RECOGNITION,
        BUTTON_1_BANKNOTES_RECOGNITION,
        BUTTON_1_VOLUNTEER_HELP,
        BUTTON_1_QR_CODES,
        BUTTON_1_COLOR_RECOGNITION,
        BUTTON_1_SOS
    };

    enum
    {
        RANGER_NEAR_FIELD = 1,
        RANGER_MIDDLE_FIELD,
        RANGER_FAR_FIELD
    };

    enum class RecognizingObjects: quint64
    {
        BUS = quint64(1),
        CAR = quint64(1) << 1,
        ORANGE = quint64(1) << 2,
        BANANA = quint64(1) << 3,
        GLASS = quint64(1) << 4,
        BOTTLE = quint64(1) << 5,
        VASE = quint64(1) << 6,
        BICYCLE = quint64(1) << 7,
        FORK = quint64(1) << 8,
        SOFA = quint64(1) << 9,
        UMBRELLA = quint64(1) << 10,
        TOOTHBRUSH = quint64(1) << 11,
        KEYBOARD = quint64(1) << 12,
        BOOK = quint64(1) << 13,
        CAT = quint64(1) << 14,
        BED = quint64(1) << 15,
        CUP = quint64(1) << 16,
        SPOON = quint64(1) << 17,
        MICROWAVE_OVEN = quint64(1) << 18,
        MOBILE_PHONE = quint64(1) << 19,
        MONITOR = quint64(1) << 20,
        CARROT = quint64(1) << 21,
        COMPUTER_MOUSE = quint64(1) << 22,
        SOFT_TOY = quint64(1) << 23,
        BALL = quint64(1) << 24,
        KNIFE = quint64(1) << 25,
        SCISSORS = quint64(1) << 26,
        LAPTOP = quint64(1) << 27,
        OVEN = quint64(1) << 28,
        REMOTE = quint64(1) << 29,
        SINK = quint64(1) << 30,
        BACKPACK = quint64(1) << 31,
        TRAFFIC_LIGHT = quint64(1) << 32,
        BENCH = quint64(1) << 33,
        DOG = quint64(1) << 34,
        TABLE = quint64(1) << 35,
        CHAIR = quint64(1) << 36,
        BAG = quint64(1) << 37,
        SANDWICH = quint64(1) << 38,
        PLATE = quint64(1) << 39,
        TOILET_HAIR = quint64(1) << 40,
        DRYER = quint64(1) << 41,
        REFRIGERATOR = quint64(1) << 42,
        FLOWER = quint64(1) << 43,
        CLOCK = quint64(1) << 44,
        MAN = quint64(1) << 45,
        SUITCASE = quint64(1) << 46,
        APPLE = quint64(1) << 47
    };

    VSmartCaneBase( unsigned id,
                    const QString& applicationName,
                    std::size_t frameSize,
                    const QString& speakersCardName,
                    unsigned playbackSamplerate,
                    const QBluetoothAddress &scanAdapter,
                    const QBluetoothAddress &audioAdapter,
                    const QString& logsFileToSend,
                    const QString& secondAudioCardName ) noexcept;

protected:
    void registerInitialParameters( quint8 powerLevel,
                                    bool sendingLogs,
                                    quint8 minFreeSpacePercent,
                                    quint8 rangerField,
                                    bool debugMode,
                                    quint8 volume,
                                    const QString& brailleDisplayName,
                                    quint64 brailleDisplayMacAddress,
                                    const QString& speakerName,
                                    quint64 speakerMacAddress,
                                    quint8 voiceType,
                                    quint16 speechSpeed,
                                    quint8 addedFacesCount,
                                    bool installUpdatesAutomatically,
                                    uint8_t updateStatus,
                                    const QString& currentVersion,
                                    const QString& updateVersion,
                                    const QString& updateDescription,
                                    bool readInstructionOnStart,
                                    const std::array<quint8,3>& buttonsFunctions,
                                    const QDateTime& lastSendLogTime,
                                    bool useEmbeddedSpeaker,
                                    quint64 recognizingObjects,
                                    const QStringList& sosPhoneNumbers,
                                    const QString& smsText ) noexcept;

    void setAddedFacesCount( quint8 phc) noexcept;
    void setAudioVolumeApp( quint8 volume) noexcept;
    void setButtonsFunctions( quint8 phc, unsigned buttonIndex) noexcept;

    virtual bool processSetRangerFieldRequested( quint8 rangerField ) noexcept = 0;
    virtual bool processSetDebugModeRequested( bool debugMode ) noexcept = 0;
    virtual bool processSetUseEmbeddedSpeakerRequested( bool use ) noexcept = 0;
    virtual bool processSetAudioVolumeRequested( quint8 volume ) noexcept = 0;
    virtual void processNewRecognitionModuleAddressReceived( const QBluetoothAddress& address ) noexcept = 0;
    virtual bool processSetVoiceTypeRequested( quint8 type ) noexcept = 0;
    virtual bool processSetSpeechSpeedRequested( quint16 speed ) noexcept = 0;
    virtual bool processSetReadInstructionOnStartRequested( bool read ) noexcept = 0;
    virtual bool processClearAddedFacesRequested( ) noexcept = 0;
    virtual void processNewImage( const QImage& image,
                                  const QString& name ) noexcept = 0;
    virtual void processNewButtonsFunctions( quint8 buttonsFunctions, unsigned buttonIndex ) noexcept = 0;
    virtual void processRestorePreviousSoftwareVersion( ) noexcept = 0;
    virtual bool processSetRecognizingObjects( quint64 recognizingObjects ) noexcept = 0;
    virtual void processSetSosPhoneNumbers( const QStringList& phoneNumbers ) noexcept = 0;
    virtual void processSmsText( const QString& text ) noexcept = 0;

    VSmartCaneBleAdapter* getBleAdapter( ) noexcept;

private slots:
    void onSetRangerFieldRequested( quint8 rangerField ) noexcept;
    void onSetDebugModeRequested( bool debugMode ) noexcept;
    void onSetUseEmbeddedSpeakerRequested( bool use ) noexcept;
    void onSetAudioVolumeRequested( quint8 volume ) noexcept;
    void onRecognitionModuleAddressGot( const QBluetoothAddress& address ) noexcept;
    void onSetVoiceTypeRequested( quint8 type ) noexcept;
    void onSetSpeechSpeedRequested( quint16 speed ) noexcept;
    void onSetReadInstructionOnStartRequested( bool read ) noexcept;
    void onClearAddedFacesRequested( ) noexcept;
    void onNewImageIdGot( const QUuid& id ) noexcept;
    void onImageRequestFinished(QNetworkReply *reply) noexcept;
    void onAuthenticationRequired(QNetworkReply *reply,
                                  QAuthenticator *authenticator) noexcept;
    void onSetButtonsFunctionsRequested( quint8 buttonsFunctions, unsigned buttonIndex ) noexcept;
    void onRestorePreviousSoftwareVersionRequested( ) noexcept;
    void onSetRecognizingObjectsRequested( quint64 recognizingObjects ) noexcept;
    void onSetSosPhoneNumbersRequested( const QStringList& phoneNumbers ) noexcept;
    void onSetNewSystemTimeRequested( quint64 newTimestamp ) noexcept;

    void onSmsTextReceived( const QString& text ) noexcept;

private:
    Q_DISABLE_COPY( VSmartCaneBase )

    QNetworkAccessManager *m_networkManager;
};

#endif
